/'
 Makro.bi
 
 This file contains a collection of many macros, functions and procs, that may be
 useful in one or another situation.
 
 It contains these routines:
   - Makros
      + d2r(x)
         ' converts deg to rad
      + r2d(x)
         ' converts rad to deg
      + p2r(x)
         ' converts percent to rad
      + Sgk(x, y)
         ' set's count of a number's decimals, but doesn't round.
            x is the number to be set, y is the count of decimals.
            negative numbers (for both parameters) are allowed, too.
            Sgk(10.52342345632, 2) = 10.52
            Sgk(5.5, 0) = 5
            Sgk(-5.5, 0) = -5
            Sgk(123, -1) = 120
      + RouU(x)
         ' rounds to next biger decimal (similar to fix)
            RouU( 5.2) = 6
            RouU(-5.2) = -5
         ' Note: RouD, which would stand for round down is not implemented, as this task is already done by
           the function INT().
      + SgkU(x, y)
         ' set's count of a numbers decimals and rounds up
      + SgkD(x, y)
         ' set's count of a numbers decimals and rounds down
         ' Note: A function, that does automatic up- or down-rounding is listed in the section 
           Functions (see below)
      + Quote(m)
         ' Add's a CHR(34) (that's ") to the beginning and end of an expression. 
            (Uses &, so the expression may be nummerical)
      + LogX(num, bas)
         ' Returns logarithm of number 'num' to base 'bas'.
            bas ^ LogX = num.
   
   - Functions
      + FileExists(file AStr) AInt
         ' Returns -1 If File exists
         // Removed, as it is implemented in freebasic, itself.
      + FolderExists(folder AStr) AInt
         ' Returns -1 If Folder exists
      + GetFileName(fileWithPath AStr) AInt
         ' Returns only filename.
      + GetPathName(fileWithPath AStr) AInt
         ' Returns only path
      + GetFiles (Pattern AStr, Attrib AInt, Files() AStr, delOld AInt = -1) AInt
         ' Fills the array Files() with a List of Files, that match with Pattern.
            For Attrib see DIR.
            If 'delold' is non-zero, Files() will be re-initialized, otherwise, the scan
            will be added to the existing data in the array.
      + Round OLoad (Number ASng/Dbl, Count AInt/Dbl) ASng/ADbl
         ' rounds to 'count' decimals, decidig if rounding up or down automatically
      + GetTextWidth (Text AStr, Font AAnyPtr = 0) AInt
         ' Returns width of a Text in Pixels, respecting line breaks and Font-Width
      + GetTextHeight (Text AStr, Font AAnyPtr = 0) AInt
         ' Returns Height of a Text in Pixels, respecting line breaks and Font-Height. 
            If no Font is specified, Symbol 'DWX_StdFontHeight' is used as Font-Heigt.
      + GetLineCount (Text AStr) AInt
         ' Returns count of CHR(13) in a Text.
      + GetTextLine (Text AStr, LineNr AInt) AStr
         ' Returns Line number 'LineNr' of a text.
            For example, GetTextLIne(Text, 2) would return the 2nd line.
      + PSwAns (Ans AStr, mxShLn AInt, Cursor AInt, Flags AInt) AInt
         ' Returns a String, that contains max. mxShLn chars, made out of Ans, beginning at char 'Cursor' of Ans.
            If 'Cursor' is zero, result will be formatted, so that it is the end of 'ans'.
            Flags is a complement of some symbols. For this Func, only DWI_Password has a meanig. It makes
            PSwAns to return a string of asterisks (*).
      + WaitForKey (NumbersOnly AInt = 0, AllowCtrlKeys AInt = 0) AStr
         ' Waits for KeyPress like GETKEY, but returns pressed key, and is able to be filtered.
      + LVector (x AInt/ASng/ADbl, y AInt/ASng/ADbl, z AInt/ASng/ADbl = 0) ADbl
         ' Returns length of an vector.
      
   - Procs
     + CopyMem (dst AAnyPtr, src AAnyPtr, count AInt)
         ' Copies a certain count of bytes from 'src' to 'dst'
            Works like MOV in ASM, but allows to copy a certain ammount of bytes
            By the way, this IS an ASM-Proc...
            Thanks to volta, who posted the ASM-Code
     + fastIntSwap (x AInt, y AInt)
         ' Works just as SWAP, but faster and for Integers only
            thanks to psygate, who postet the ASM-Code
     + SortArray ( Array () AInt )
         ' Sorts the elements of an array.
            Works only for one-Dim-Integer-Arrays, but may be modified easily.
     + AddToArray OLoad  (Array () AInt, Element AInt =  0, Idx AInt = -1 )
         ' Adds an element to an array at any index, you wish.
            Also in String-Version available.
            If Idx is -1, the new element is added at the end of array.
'/

'=============================================================================='

'  Angle
#IfNDef pi
   #Define pi ( 4# * Atn(1#) )
#EndIf

#Define d2r(x) ((x) * pi / 180)
#Define r2d(x) ((x) / pi * 180)
#Define p2r(x) ((x - .25) * 2 * pi)
'------------------------------------------------------------------------------'
'  Decimals
#Define Sgk(x, y) ( Fix(x * (10 ^ y)) / (10 ^ y) )
#Define RouU(x) ( -Int(-x) )
#Define SgkU(x, y) ( CSng(RouU(x * (10 ^ y))) / (10 ^ y) )
#Define SgkD(x, y) ( CSng(Int(x * (10 ^ y))) / (10 ^ y) )
'------------------------------------------------------------------------------'
' Misc
#IfNDef kc_QM
   #Define kc_QM Chr(34)
#EndIf
#Define Quote(m) (kc_QM & m & kc_QM)

#Define LogX(num, bas) ( Log(num) / Log(bas) )
'------------------------------------------------------------------------------'
#IfNDef CommonInc_Constants
   #Define FKey                Chr(255)
   #Define kc_Esc              Chr( 27)
   #Define kc_Back             Chr(  8)
   #Define kc_Tab              Chr(  9)
   #Define kc_enter            Chr( 13)
   #Define kc_Left      FKey + Chr( 75)   ' K
   #Define kc_Right     FKey + Chr( 77)   ' M
   #Define kc_Up        FKey + Chr( 72)   ' H
   #Define kc_Down      FKey + Chr( 80)   ' P
   #Define kc_Pos1      FKey + Chr( 71)   ' G
   #Define kc_End       FKey + Chr( 79)   ' O
   #Define kc_Ins       FKey + Chr( 82)   ' R
   #Define kc_Del       FKey + Chr( 83)   ' S
#EndIf
'=============================================================================='
' Functions & Procs
'DFunc FileExists  (File   AStr) AInt
DFunc FolderExists(Folder AStr) AInt
DFunc GetFileName (FileWithPath AStr) AStr
DFunc GetPathName (FileWithPath AStr) AStr
DFunc GetFiles (Pattern AStr, Attrib AInt, Files() AStr, delOld AInt = -1) AInt

DFunc Round OLoad (Number ASng, Count AInt) ASng
DFunc Round       (Number ADbl, Count AInt) ADbl

DProc SortArray OLoad   (Array () AInt )
DProc SortArray         (Array () AStr )
DProc SortArray         (Array () ADbl )

DProc AddToArray OLoad  (Array () AInt, Element AInt =  0, Idx AInt = -1 )
DProc AddToArray        (Array () AStr, Element AStr = "", Idx AInt = -1 )
DProc AddToArray        (Array () ADbl, Element ADbl =  0, Idx AInt = -1 )

DProc fastIntSwap (x AInt, y AInt)

DFunc GetTextWidth   (Text AStr, Font AAnyPtr = 0) AInt
DFunc GetTextHeight  (Text AStr, Font AAnyPtr = 0) AInt
DFunc GetLineCount   (Text AStr) AInt
DFunc GetTextLine    (Text AStr, LineNr AInt) AStr
DFunc PSwAns         (Ans AStr, mxShLn AInt, Cursor AInt, Flags AInt) AStr

#Define DWI_NumbersOnly    &H01
#Define DWI_Password       &H02
#Define DWI_NoVisCursor    &H04
#Define DWX_Bold           &H10
#Define DWX_Underlined     &H20
#Define DWX_StdFontHeight  16
/'
These Symbols are here, because the are used in more files; each of these files uses
Func PSwAns, which is in this file.
'/

DProc CopyMem (destination AAnyPtr, source AAnyPtr, count AInt)
DFunc WaitForKey (NumbersOnly AInt = 0, AllowCtrlKeys AInt = 0) AStr
DFunc LVector OLoad (x AInt, y AInt, z AInt = 0) ADbl
DFunc LVector       (x ASng, y ASng, z ASng = 0) ADbl
DFunc LVector       (x ADbl, y ADbl, z ADbl = 0) ADbl
'------------------------------------------------------------------------------'
'Func FileExists(File AStr) AInt
'   DInt RetVal, f
'   
'   f = FreeFile
'   RetVal = Open(File For Input As #f)
'   If RetVal = 0 Then Close #f
'   
'   Function = (RetVal = 0)
'EFunc
'..............................................................................'
Func FolderExists (Path As String) AInt
   DStr last
   last = CurDir
   ChDir Path
   If CurDir = Path              Then Func = -1   
   If Curdir = last + "\" + Path Then Func = -1
   ChDir last
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func GetFileName (FWP AStr) AStr
   DInt i, l
   
   Do
      l = i
      i = Instr(i + 1, FWP, "\")
   Loop While i
   
   Function = Right(FWP, Len(FWP) - l)
EFunc
'..............................................................................'
Func GetPathName (FWP AStr) AStr
   DInt i, l
   
   Do
      l = i
      i = Instr(i + 1, FWP, "\")
   Loop While i
   
   Function = Left(FWP, l - 1)
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func GetFiles (Pattern AStr, Attrib AInt, Files() AStr, delOld AInt = -1) AInt
   DInt i = 1
   DStr tmp
   
   tmp = Dir(Pattern, Attrib)
   If tmp = "." Then tmp = Dir("", Attrib) : tmp = Dir("", Attrib)
   If Len(tmp) = 0 Then Return 0
   
   If delOld Then 
      Redim AStr Files (1 To 1)
      delOld = 0
   Else 
      delOld = UBound(Files)
      i = delOld + 1
      RePre AStr Files (1 To i)
   EIf
   
   Do
      Files(i) = tmp
      tmp = Dir("", Attrib)
      If Len(tmp) Then 
         i += 1
         RePre AStr Files(1 To i)
      Else
         QDo
      EIf
   Loop
   
   Function = i - delOld
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func Round (Number ASng, Count AInt) ASng
   DInt last, tmp
   DSng s
   
   s = Sgn(Number)
   Number = Abs(Number)
   
   tmp  = Fix(Number * ( 10 ^  Count      )) * 10
   last = Fix(Number * ( 10 ^ (Count + 1) )) - tmp
   
   If last >= 5 Then Return s * SgkU(Number, Count) Else Return s * SgkD(Number, Count)
EFunc
'..............................................................................'
Func Round (Number ADbl, Count AInt) ADbl
   DInt last, tmp
   DDbl s
   
   s = Sgn(Number)
   Number = Abs(Number)
   
   tmp  = Fix(Number * ( 10 ^ Count       )) * 10
   last = Fix(Number * ( 10 ^ (Count + 1) )) - tmp
   
   If last >= 5 Then Return s * SgkU(Number, Count) Else Return s * SgkD(Number, Count)
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Proc CopyMem (Ziel AAnyPtr, Quelle AAnyPtr, Bytes AInt) 
   Asm 
      mov edi, dword Ptr [Ziel] 
      mov esi, dword Ptr [Quelle] 
      mov ecx, dword Ptr [Bytes] 
      
      shr ecx, 1              ' if dividable by 2 then...
      jnc CopyMem_word        ' Copy words (2bytes)
      movsb       ' move one byte of String from esi to edi
      
      CopyMem_word: 
         shr ecx, 1           ' if dividable by 4 then...
         jnc CopyMem_dword    ' Copy double-Words (4bytes)
         movsw    ' move two bytes of String from esi to edi
         
      CopyMem_dword: 
         shr ecx, 1           ' if dividable by 8 then...
         jnc CopyMem_qwords   ' Copy quadro-Words (8bytes)
         movsd    ' move four bytes of String from esi to edi
         
      CopyMem_qwords: 
         jecxz CopyMem_end    ' if string is copied fully then qproc
         
      CopyMem_loop: 
         movq mm0, [esi]      ' move eight bytes from esi into mm0
         movq [edi], mm0      ' move eight bytes out of mm0 at edi
         lea esi, [esi + 8]   ' update source-Adress: Add 8
         lea edi, [edi + 8]   ' update destination-Adress: Add 8
         dec ecx              ' allowed for eight bytes as Bytes was divided by 8. (SHR's)
         jnz CopyMem_loop     ' If ecx > 0 Then Goto CopyMem_Loop
         
      CopyMem_end: 
         emms 
   EAsm
EProc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func WaitForKey (NumbersOnly AInt = 0, allowCtrlKeys AInt = 0) AStr
   DStr key
   
   WaitForKeyNoNumber:
   Do : key = Inkey : Sleep 1 : Loop Until Len(key)
   If NumbersOnly Then 
      If Not (_
              (Asc(Key) >= 48) And (Asc(Key) <= 57) Or _    ' numbers
              (Asc(Key)  = 46)                      Or _    ' point (decimal)
              (Asc(Key)  = 45)                      Or _    ' minus
              (Asc(Key)  = 43)                       ) _    ' plus
              Then 
         If Not allowCtrlKeys Then
            Jmp WaitForKeyNoNumber
         Else
            If (Key <> kc_enter) And _
               (Key <> kc_esc  ) And _
               (Key <> kc_back ) And _
               (Key <> kc_pos1 ) And _
               (Key <> kc_end  ) And _
               (Key <> kc_Up   ) And _
               (Key <> kc_Down ) And _
               (Key <> kc_Left ) And _
               (Key <> kc_Right) And _
               (Key <> kc_Del  ) And _
               (Key <> kc_Ins  )     _
               Then Jmp WaitForKeyNoNumber
         EIf
      EIf
   EIf
   Return Key
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Proc fastIntSwap (x AInt, y AInt)
   x Xor= y 
   y Xor= x 
   x Xor= y 
EProc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Proc SortArray( Array() AInt )
   DInt i, j, count
   
   count = UBound(Array)
   For i = 1 To count - 1
      For j = i + 1 To count
         
         If Array(j) < Array(i) Then FastIntSwap Array(j), Array(i)
         
      Next
   Next
EProc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Proc AddToArray (Array () AStr, Element AStr = "", Idx AInt = -1 )
   DInt i, u, l
   
   u = UBnd(Array) + 1
   l = LBnd(Array)
   If idx = -1 Then idx = l
   
   RePre AStr Array( l To u )
   
   For i = u To Idx + 1
      Array(i) = Array(i - 1)
   Next
   
   Array(idx) = Element
EProc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func PSwAns (Ans AStr, mxShLn AInt, Cursor AInt, Flags AInt) AStr
   DStr Re
   DInt sw
   
   If Len(Ans) = 0 Then Return ""
   
   Re = Ans
   If (Len(Ans) > mxShLn) And (mxShLn <> 0) Then 
      sw = Cursor - mxShLn
      If sw < 1 Then sw = 1
      Re = Mid(Ans, sw, mxShLn)
   EIf
   
   #IfDef CommonInc_DWI_Flags
      If Flags And DWI_Password Then Re = String(Len(Re), "*")
   #EndIf
   
   Function = Re
EFunc
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::'
Func GetTextWidth (Text AStr, Fnt AAnyPtr = 0) AInt
   DInt i, w, a, first, last, maxline
   DUBytPtr Font
   
   Font = Cast(UByte Ptr, Fnt)
   
   If Font Then
         first = Font[5]
         last  = Font[6]
   EIf
   
   
   For i = 0 To Len(Text) - 1
      a = Text[i]
      
      If a = 13 Then
         If w > maxline Then maxline = w
         w = 0
         Continue For
      EIf
      
      If Font Then
         If (a >= first) And (a <= last) Then
            w += Font[7 + a - first]
         EIf
      Else
         w += 8
      EIf
   Next
   
   If maxLine = 0 Then maxLine = w
   
   Return maxLine
EFunc
'..............................................................................'
Func GetTextHeight (Text AStr, Fnt AAnyPtr = 0) AInt
   DInt i, l = 1, h
   DUBytPtr Font
   
   Font = Cast(UByte Ptr, Fnt)
   
   For i = 0 To Len(Text) -1
      If Text[i] = 13 Then l += 1
   Next
   
   If Font Then
      h = (Font[3] Shl 8 Or Font[2]) - 1
   Else
      h = DWX_StdFontHeight
   EIf
   
   Return l * h
EFunc
'..............................................................................'
Func GetLineCount (Text AStr) AInt
   DInt i, l = 1
   
   For i = 0 To Len(Text) -1
      If Text[i] = 13 Then l += 1
   Next
   
   Return l
EFunc
'..............................................................................'
Func GetTextLine (Text AStr, LineNr AInt) AStr
   DInt i, h = 1
   DStr RetVal
   
   For i = 0 To Len(Text) - 1
      If Text[i] = 13 Then
         h += 1
      ElseIf h = LineNr Then 
         RetVal += Chr$(Text[i])
      ElseIf h > LineNr Then 
         Return RetVal
      EIf
   Next
   
   Return RetVal
EFunc
'=============================================================================='